"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTemplateParamObjectFromArgs = exports.CommunityCreateCommand = void 0;
const core_1 = require("@salesforce/core");
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const ConnectExecutor_1 = require("../../shared/connect/services/ConnectExecutor");
const CommunityCreateResource_1 = require("../../shared/community/connect/CommunityCreateResource");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-community', 'create');
const MESSAGE_KEY = 'message';
const NAME_KEY = 'name';
const JOBID_KEY = 'jobId';
const ACTION_KEY = 'action';
/**
 * A command to create a community.
 * This is just an sfdx wrapper around the community create connect endpoint
 */
class CommunityCreateCommand extends sf_plugins_core_1.SfCommand {
    async run() {
        const { flags, argv, args } = await this.parse(CommunityCreateCommand);
        const templateParams = (0, exports.getTemplateParamObjectFromArgs)((0, sf_plugins_core_1.parseVarArgs)(args, argv));
        const createCommand = new CommunityCreateResource_1.CommunityCreateResource({
            name: flags.name,
            urlPathPrefix: flags['url-path-prefix'],
            templateName: flags['template-name'],
            description: flags.description,
            templateParams,
        });
        return new ConnectExecutor_1.ConnectExecutor(createCommand, flags['target-org'].getConnection(flags['api-version']))
            .callConnectApi()
            .then((results) => {
            this.displayResults(results);
            return results;
        });
    }
    displayResults(results) {
        const columns = {
            [NAME_KEY]: { header: 'Name' },
            [MESSAGE_KEY]: { header: 'Message' },
            [JOBID_KEY]: { header: 'JobId' },
            [ACTION_KEY]: { header: 'Action' },
        };
        this.styledHeader(messages.getMessage('response.styleHeader'));
        this.table([results], columns, { 'no-truncate': true });
    }
}
exports.CommunityCreateCommand = CommunityCreateCommand;
CommunityCreateCommand.deprecateAliases = true;
CommunityCreateCommand.aliases = ['force:community:create'];
CommunityCreateCommand.summary = messages.getMessage('summary');
CommunityCreateCommand.description = messages.getMessage('description');
CommunityCreateCommand.examples = messages.getMessages('examples');
CommunityCreateCommand.strict = false;
CommunityCreateCommand.flags = {
    name: sf_plugins_core_1.Flags.string({
        char: 'n',
        summary: messages.getMessage('flags.name.summary'),
        required: true,
    }),
    'template-name': sf_plugins_core_1.Flags.string({
        char: 't',
        summary: messages.getMessage('flags.templateName.summary'),
        description: messages.getMessage('flags.templateName.description'),
        required: true,
        deprecateAliases: true,
        aliases: ['templatename'],
    }),
    'url-path-prefix': sf_plugins_core_1.Flags.string({
        char: 'p',
        summary: messages.getMessage('flags.urlPathPrefix.summary'),
        description: messages.getMessage('flags.urlPathPrefix.description'),
        // The api requires you to pass this, it accepts an empty string
        default: '',
        deprecateAliases: true,
        aliases: ['urlpathprefix'],
    }),
    description: sf_plugins_core_1.Flags.string({
        char: 'd',
        summary: messages.getMessage('flags.description.summary'),
        description: messages.getMessage('flags.description.description'),
    }),
    'target-org': sf_plugins_core_1.requiredOrgFlagWithDeprecations,
    loglevel: sf_plugins_core_1.loglevel,
    'api-version': sf_plugins_core_1.orgApiVersionFlagWithDeprecations,
};
const getTemplateParamObjectFromArgs = (args) => {
    // make sure there's nothing bad
    const badArgs = Object.keys(args).filter((key) => !key.startsWith('templateParams'));
    if (badArgs.length) {
        throw new core_1.SfError(`Invalid argument(s): ${badArgs.join(', ')}`, 'InvalidArgument', [
            'Arguments should start with templateParams, like templateParams.AuthenticationType=UNAUTHENTICATED',
        ]);
    }
    // construct the return object
    const templateParams = Object.fromEntries(Object.entries(args).map(([key, value]) => [key.replace('templateParams.', ''), value]));
    return templateParams;
};
exports.getTemplateParamObjectFromArgs = getTemplateParamObjectFromArgs;
//# sourceMappingURL=create.js.map