"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const core_1 = require("@salesforce/core");
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const flags_1 = require("../../../shared/flags");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-org', 'delete_sandbox');
class EnvDeleteSandbox extends sf_plugins_core_1.SfCommand {
    async run() {
        const flags = (await this.parse(EnvDeleteSandbox)).flags;
        const username = flags['target-org'];
        const orgId = (await core_1.AuthInfo.create({ username })).getFields().orgId;
        const isSandbox = await (await core_1.StateAggregator.getInstance()).sandboxes.hasFile(orgId);
        if (!isSandbox) {
            throw messages.createError('error.isNotSandbox', [username]);
        }
        if (flags['no-prompt'] || (await this.confirm(messages.getMessage('prompt.confirm', [username])))) {
            try {
                const org = await core_1.Org.create({ aliasOrUsername: username });
                await org.delete();
                this.logSuccess(messages.getMessage('success', [username]));
            }
            catch (e) {
                if (e instanceof Error && e.name === 'DomainNotFoundError') {
                    // the org has expired, so remote operations won't work
                    // let's clean up the files locally
                    const authRemover = await core_1.AuthRemover.create();
                    await authRemover.removeAuth(username);
                    this.logSuccess(messages.getMessage('success.Idempotent', [username]));
                }
                else if (e instanceof Error && e.name === 'SandboxNotFound') {
                    this.logSuccess(messages.getMessage('success.Idempotent', [username]));
                }
                else {
                    throw e;
                }
            }
        }
        return { username, orgId };
    }
}
exports.default = EnvDeleteSandbox;
EnvDeleteSandbox.summary = messages.getMessage('summary');
EnvDeleteSandbox.description = messages.getMessage('description');
EnvDeleteSandbox.examples = messages.getMessages('examples');
EnvDeleteSandbox.aliases = ['env:delete:sandbox'];
EnvDeleteSandbox.deprecateAliases = true;
EnvDeleteSandbox.flags = {
    'target-org': (0, flags_1.orgThatMightBeDeleted)({
        summary: messages.getMessage('flags.target-org.summary'),
        required: true,
    }),
    'no-prompt': sf_plugins_core_1.Flags.boolean({
        char: 'p',
        summary: messages.getMessage('flags.no-prompt.summary'),
    }),
};
//# sourceMappingURL=sandbox.js.map