"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveUsername = exports.targetUsernameFlag = exports.getCoverageFormattersOptions = exports.createCoverageReport = exports.DeployCommand = exports.reportsFormatters = void 0;
const path = require("path");
const fs = require("fs");
const source_deploy_retrieve_1 = require("@salesforce/source-deploy-retrieve");
const core_1 = require("@salesforce/core");
const ts_types_1 = require("@salesforce/ts-types");
const kit_1 = require("@salesforce/kit");
const apex_node_1 = require("@salesforce/apex-node");
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const sourceCommand_1 = require("./sourceCommand");
const stash_1 = require("./stash");
const coverageUtils_1 = require("./coverageUtils");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-source', 'deployCommand');
exports.reportsFormatters = Object.keys(apex_node_1.DefaultReportOptions);
class DeployCommand extends sourceCommand_1.SourceCommand {
    constructor() {
        super(...arguments);
        this.displayDeployId = (0, kit_1.once)((id) => {
            if (!this.jsonEnabled()) {
                this.log(`Deploy ID: ${id}`);
            }
        });
        this.isRest = false;
        this.isAsync = false;
        this.updateDeployId = (0, kit_1.once)((id) => {
            this.displayDeployId(id);
            const stashKey = stash_1.Stash.getKey(this.id);
            stash_1.Stash.set(stashKey, { jobid: id });
        });
    }
    /**
     * Request a report of an in-progress or completed deployment.
     *
     * @param id the Deploy ID of a deployment request
     * @returns DeployResult
     */
    async report(conn, id) {
        const deployId = this.resolveDeployId(id);
        this.displayDeployId(deployId);
        const res = await conn.metadata.checkDeployStatus(deployId, true);
        const deployStatus = res;
        const componentSet = this.componentSet || new source_deploy_retrieve_1.ComponentSet();
        return new source_deploy_retrieve_1.DeployResult(deployStatus, componentSet);
    }
    /**
     * Checks the response status to determine whether the deploy was successful.
     * Async deploys are successful unless an error is thrown, which resolves as
     * unsuccessful in oclif.
     */
    resolveSuccess() {
        const StatusCodeMap = new Map([
            [source_deploy_retrieve_1.RequestStatus.Succeeded, 0],
            [source_deploy_retrieve_1.RequestStatus.Canceled, 1],
            [source_deploy_retrieve_1.RequestStatus.Failed, 1],
            [source_deploy_retrieve_1.RequestStatus.SucceededPartial, 68],
            [source_deploy_retrieve_1.RequestStatus.InProgress, 69],
            [source_deploy_retrieve_1.RequestStatus.Pending, 69],
            [source_deploy_retrieve_1.RequestStatus.Canceling, 69],
        ]);
        if (!this.isAsync) {
            this.setExitCode(StatusCodeMap.get(this.deployResult.response?.status) ?? 1);
        }
    }
    /**
     * This method is here to provide a workaround to stubbing a constructor in the tests.
     *
     * @param id
     */
    // eslint-disable-next-line class-methods-use-this
    createDeploy(conn, id) {
        return new source_deploy_retrieve_1.MetadataApiDeploy({ usernameOrConnection: conn.getUsername(), id });
    }
    resolveDeployId(id) {
        if (id) {
            return id;
        }
        else {
            const stash = stash_1.Stash.get(stash_1.Stash.getKey(this.id));
            if (!stash) {
                throw new core_1.SfError(messages.getMessage('MissingDeployId'));
            }
            return stash.jobid;
        }
    }
    // SOAP is the default unless:
    //   1. SOAP is specified with the soapdeploy flag on the command
    //   2. The restDeploy SFDX config setting is explicitly true.
    isRestDeploy(soapdeploy = true) {
        if (soapdeploy === true) {
            this.debug('soapdeploy flag === true.  Using SOAP');
            return false;
        }
        const restDeployConfig = this.configAggregator.getInfo('org-metadata-rest-deploy').value;
        // aggregator property values are returned as strings
        if (restDeployConfig === 'false') {
            this.debug('restDeploy SFDX config === false.  Using SOAP');
            return false;
        }
        else if (restDeployConfig === 'true') {
            this.debug('restDeploy SFDX config === true.  Using REST');
            return true;
        }
        else {
            this.debug('soapdeploy flag unset. restDeploy SFDX config unset.  Defaulting to SOAP');
        }
        return false;
    }
    async poll(connection, deployId, options = { wait: kit_1.Duration.days(7) }) {
        const waitDuration = options.wait;
        const defaultOptions = {
            frequency: options?.frequency ?? kit_1.Duration.seconds(1),
            timeout: options?.timeout ?? waitDuration,
            poll: async () => {
                const deployResult = await this.report(connection, deployId);
                return {
                    completed: (0, ts_types_1.getBoolean)(deployResult, 'response.done'),
                    payload: deployResult,
                };
            },
        };
        const pollingOptions = { ...defaultOptions, ...options };
        const pollingClient = await core_1.PollingClient.create(pollingOptions);
        return pollingClient.subscribe();
    }
    async deployRecentValidation(id, conn) {
        const validatedDeployId = await conn.metadata.deployRecentValidation({ id, rest: this.isRest });
        // This is the deploy ID of the deployRecentValidation response, not
        // the already validated deploy ID (i.e., validateddeployrequestid).
        this.updateDeployId(validatedDeployId);
        this.asyncDeployResult = { id: validatedDeployId };
        return this.isAsync ? this.report(conn, validatedDeployId) : this.poll(conn, validatedDeployId);
    }
    maybeCreateRequestedReports(options) {
        // only generate reports if test results are present
        if (this.deployResult.response?.numberTestsTotal) {
            if (options.coverageformatters) {
                (0, exports.createCoverageReport)(this.deployResult, options.coverageformatters, 'no-map', this.resultsDir);
            }
            if (options.junit) {
                this.createJunitResults(this.deployResult, options.org);
            }
        }
    }
    createJunitResults(deployResult, org) {
        const testResult = (0, coverageUtils_1.transformDeployTestsResultsToTestResult)(org.getConnection(), deployResult.response?.details?.runTestResult);
        if (testResult.summary.testsRan > 0) {
            const jUnitReporter = new apex_node_1.JUnitReporter();
            const junitResults = jUnitReporter.format(testResult);
            const junitReportPath = path.join(this.resultsDir, 'junit');
            fs.mkdirSync(junitReportPath, { recursive: true });
            fs.writeFileSync(path.join(junitReportPath, 'junit.xml'), junitResults, 'utf8');
        }
    }
    // eslint-disable-next-line class-methods-use-this
    resolveOutputDir(coverageFormatters, junit, resultsDir, deployId, noThrow) {
        if (resultsDir) {
            return resultsDir;
        }
        if (coverageFormatters || junit) {
            if (deployId) {
                return deployId;
            }
            if (!noThrow) {
                throw new core_1.SfError(messages.getMessage('resultsDirMissing'));
            }
        }
        return undefined;
    }
}
exports.DeployCommand = DeployCommand;
const createCoverageReport = (deployResult, formatters, sourceDir, resultsDir) => {
    const apexCoverage = (0, coverageUtils_1.transformCoverageToApexCoverage)((0, kit_1.ensureArray)(deployResult.response?.details?.runTestResult?.codeCoverage));
    fs.mkdirSync(resultsDir, { recursive: true });
    const options = (0, exports.getCoverageFormattersOptions)(formatters);
    const coverageReport = new apex_node_1.CoverageReporter(apexCoverage, resultsDir, sourceDir, options);
    coverageReport.generateReports();
};
exports.createCoverageReport = createCoverageReport;
const getCoverageFormattersOptions = (formatters = []) => {
    const reportFormats = formatters;
    const reportOptions = Object.fromEntries(reportFormats.map((format) => {
        const formatDefaults = apex_node_1.DefaultReportOptions[format];
        return [
            format,
            {
                ...formatDefaults,
                // always join any subdir from the defaults with our custom coverage dir
                ...('subdir' in formatDefaults ? { subdir: path.join('coverage', formatDefaults.subdir) } : {}),
                // if there is no subdir, we also put the file in the coverage dir, otherwise leave it alone
                ...('file' in formatDefaults && !('subdir' in formatDefaults)
                    ? { file: path.join('coverage', formatDefaults.file) }
                    : {}),
            },
        ];
    }));
    return {
        reportFormats,
        reportOptions,
    };
};
exports.getCoverageFormattersOptions = getCoverageFormattersOptions;
exports.targetUsernameFlag = sf_plugins_core_1.Flags.string({
    required: true,
    char: 'u',
    deprecateAliases: true,
    // DO NOT alias to 'o', it will conflict with '--ignoreerrors'
    aliases: ['targetusername'],
    summary: messages.getMessage('flags.targetusername.summary'),
    parse: async (input) => (0, exports.resolveUsername)(input),
    default: async () => (0, exports.resolveUsername)(),
    defaultHelp: async () => (0, exports.resolveUsername)(),
});
const resolveUsername = async (usernameOrAlias) => {
    const stateAggregator = await core_1.StateAggregator.getInstance();
    // we have a value, but don't know if it's a username or an alias
    if (usernameOrAlias)
        return stateAggregator.aliases.resolveUsername(usernameOrAlias);
    // we didn't get a value, so let's see if the config has a default target org
    const configAggregator = await core_1.ConfigAggregator.create();
    const defaultUsernameOrAlias = configAggregator.getPropertyValue('target-org');
    if (defaultUsernameOrAlias)
        return stateAggregator.aliases.resolveUsername(defaultUsernameOrAlias);
    throw new core_1.SfError(messages.getMessage('missingUsername'), 'MissingUsernameError');
};
exports.resolveUsername = resolveUsername;
//# sourceMappingURL=deployCommand.js.map