"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const flags_1 = require("../../../flags");
const dataCommand_1 = require("../../../dataCommand");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-data', 'record.update');
const commonMessages = core_1.Messages.loadMessages('@salesforce/plugin-data', 'messages');
class Update extends sf_plugins_core_1.SfCommand {
    async run() {
        const { flags } = await this.parse(Update);
        this.spinner.start('Updating Record');
        let status = 'Success';
        const conn = flags['use-tooling-api']
            ? flags['target-org'].getConnection(flags['api-version']).tooling
            : flags['target-org'].getConnection(flags['api-version']);
        // oclif isn't smart of enough to know that if record-id is not set, then where is set
        const sObjectId = flags['record-id'] ?? (await (0, dataCommand_1.query)(conn, flags.sobject, flags.where)).Id;
        try {
            const updateObject = { ...(0, dataCommand_1.stringToDictionary)(flags.values), Id: sObjectId };
            const result = await conn.sobject(flags.sobject).update(updateObject);
            if (result.success) {
                this.log(messages.getMessage('updateSuccess', [sObjectId]));
            }
            else {
                const errors = (0, dataCommand_1.collectErrorMessages)(result);
                this.error(messages.getMessage('updateFailure', [errors]));
            }
            this.spinner.stop(status);
            return result;
        }
        catch (err) {
            status = 'Failed';
            this.spinner.stop(status);
            if (isSaveResult(err)) {
                throw new core_1.SfError(messages.getMessage('updateFailureWithFields', [err.errorCode, err.message, (err.fields ?? []).join(',')]));
            }
            else {
                throw err;
            }
        }
    }
}
exports.default = Update;
Update.summary = messages.getMessage('summary');
Update.description = messages.getMessage('description');
Update.examples = messages.getMessages('examples');
Update.aliases = ['force:data:record:update'];
Update.deprecateAliases = true;
Update.flags = {
    ...flags_1.orgFlags,
    sobject: sf_plugins_core_1.Flags.string({
        char: 's',
        required: true,
        summary: messages.getMessage('flags.sobject'),
        aliases: ['sobjecttype'],
        deprecateAliases: true,
    }),
    // eslint-disable-next-line sf-plugin/id-flag-suggestions
    'record-id': sf_plugins_core_1.Flags.salesforceId({
        char: 'i',
        length: 'both',
        summary: messages.getMessage('flags.recordId'),
        exactlyOne: ['where', 'record-id'],
        aliases: ['sobjectid'],
        deprecateAliases: true,
    }),
    where: sf_plugins_core_1.Flags.string({
        char: 'w',
        summary: messages.getMessage('flags.where'),
        exactlyOne: ['where', 'record-id'],
    }),
    values: sf_plugins_core_1.Flags.string({
        char: 'v',
        required: true,
        summary: messages.getMessage('flags.values'),
    }),
    'use-tooling-api': sf_plugins_core_1.Flags.boolean({
        char: 't',
        summary: messages.getMessage('flags.useToolingApi'),
        aliases: ['usetoolingapi'],
        deprecateAliases: true,
    }),
    perflog: sf_plugins_core_1.Flags.boolean({
        summary: commonMessages.getMessage('perfLogLevelOption'),
        hidden: true,
        deprecated: {
            version: '57',
        },
    }),
};
const isSaveResult = (error) => {
    const se = error;
    return Boolean(se.fields && se.errorCode && se.message);
};
//# sourceMappingURL=record.js.map