"use strict";
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const flags_1 = require("../../../shared/flags");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-org', 'delete_scratch');
class EnvDeleteScratch extends sf_plugins_core_1.SfCommand {
    async run() {
        const flags = (await this.parse(EnvDeleteScratch)).flags;
        const resolvedUsername = flags['target-org'];
        const orgId = (await core_1.AuthInfo.create({ username: resolvedUsername })).getFields().orgId;
        if (flags['no-prompt'] || (await this.confirm(messages.getMessage('prompt.confirm', [resolvedUsername])))) {
            try {
                const org = await core_1.Org.create({ aliasOrUsername: resolvedUsername });
                await org.delete();
                this.logSuccess(messages.getMessage('success', [org.getUsername()]));
                return { username: org.getUsername(), orgId: org.getOrgId() };
            }
            catch (e) {
                if (e instanceof Error && e.name === 'DomainNotFoundError') {
                    // the org has expired, so remote operations won't work
                    // let's clean up the files locally
                    const authRemover = await core_1.AuthRemover.create();
                    await authRemover.removeAuth(resolvedUsername);
                    this.logSuccess(messages.getMessage('success', [resolvedUsername]));
                }
                else if (e instanceof Error && e.name === 'ScratchOrgNotFound') {
                    this.logSuccess(messages.getMessage('success.Idempotent', [resolvedUsername]));
                }
                else {
                    throw e;
                }
            }
        }
        return { username: resolvedUsername, orgId };
    }
}
exports.default = EnvDeleteScratch;
EnvDeleteScratch.summary = messages.getMessage('summary');
EnvDeleteScratch.description = messages.getMessage('description');
EnvDeleteScratch.examples = messages.getMessages('examples');
EnvDeleteScratch.aliases = ['env:delete:scratch'];
EnvDeleteScratch.deprecateAliases = true;
EnvDeleteScratch.flags = {
    'target-org': (0, flags_1.orgThatMightBeDeleted)({
        summary: messages.getMessage('flags.target-org.summary'),
        required: true,
    }),
    'no-prompt': sf_plugins_core_1.Flags.boolean({
        char: 'p',
        summary: messages.getMessage('flags.no-prompt.summary'),
    }),
};
//# sourceMappingURL=scratch.js.map